@@ -38,7 +38,10 @@ from botocore import (
     translate,
     waiter,
 )
-from botocore.compat import HAS_CRT, MutableMapping
+from botocore.compat import (
+    HAS_CRT,  # noqa: F401
+    MutableMapping,
+)
 from botocore.configprovider import (
     BOTOCORE_DEFAUT_SESSION_VARIABLES,
     ConfigChainFactory,
@@ -48,6 +51,7 @@ from botocore.configprovider import (
     SmartDefaultsConfigStoreFactory,
     create_botocore_default_config_mapping,
 )
+from botocore.context import get_context, with_current_context
 from botocore.errorfactory import ClientExceptionsFactory
 from botocore.exceptions import (
     ConfigNotFound,
@@ -64,6 +68,7 @@ from botocore.hooks import (
 from botocore.loaders import create_loader
 from botocore.model import ServiceModel
 from botocore.parsers import ResponseParserFactory
+from botocore.plugin import get_botocore_plugins, load_client_plugins
 from botocore.regions import EndpointResolver
 from botocore.useragent import UserAgentString
 from botocore.utils import (
@@ -72,9 +77,6 @@ from botocore.utils import (
     validate_region_name,
 )
 
-from botocore.compat import HAS_CRT  # noqa
-
-
 logger = logging.getLogger(__name__)
 
 
@@ -520,7 +522,7 @@ class Session:
             ).load_credentials()
         return self._credentials
 
-    def get_auth_token(self):
+    def get_auth_token(self, **kwargs):
         """
         Return the :class:`botocore.tokens.AuthToken` object associated with
         this session. If the authorization token has not yet been loaded, this
@@ -528,8 +530,15 @@ class Session:
         return the cached authorization token.
 
         """
+        provider = self._components.get_component('token_provider')
+
+        signing_name = kwargs.get('signing_name')
+        if signing_name is not None:
+            auth_token = provider.load_token(signing_name=signing_name)
+            if auth_token is not None:
+                return auth_token
+
         if self._auth_token is None:
-            provider = self._components.get_component('token_provider')
             self._auth_token = provider.load_token()
         return self._auth_token
 
@@ -834,6 +843,7 @@ class Session:
     def lazy_register_component(self, name, component):
         self._components.lazy_register_component(name, component)
 
+    @with_current_context()
     def create_client(
         self,
         service_name,
@@ -1000,6 +1010,8 @@ class Session:
             config_store=config_store,
         )
 
+        user_agent_creator.set_client_features(get_context().features)
+
         client_creator = botocore.client.ClientCreator(
             loader,
             endpoint_resolver,
@@ -1011,6 +1023,7 @@ class Session:
             exceptions_factory,
             config_store,
             user_agent_creator=user_agent_creator,
+            auth_token_resolver=self.get_auth_token,
         )
         client = client_creator.create_client(
             service_name=service_name,
@@ -1027,6 +1040,7 @@ class Session:
         monitor = self._get_internal_component('monitor')
         if monitor is not None:
             monitor.register(client.meta.events)
+        self._register_client_plugins(client)
         return client
 
     def _resolve_region_name(self, region_name, config):
@@ -1152,6 +1166,24 @@ class Session:
             credential_inputs.append('aws_account_id')
         return ', '.join(credential_inputs) if credential_inputs else None
 
+    def _register_client_plugins(self, client):
+        plugins_list = get_botocore_plugins()
+        if plugins_list == "DISABLED" or not plugins_list:
+            return
+
+        client_plugins = {}
+        for plugin in plugins_list.split(','):
+            try:
+                name, module = [part.strip() for part in plugin.split('=')]
+                client_plugins[name] = module
+            except ValueError:
+                logger.warning(
+                    f"Invalid plugin format: {plugin}. Expected 'name=module'"
+                )
+
+        if client_plugins:
+            load_client_plugins(client, client_plugins)
+
 
 class ComponentLocator:
     """Service locator for session components."""
